#include "levellib.h"

#include <stdio.h>
#include <ctype.h>

void decode_rcomp(FILE *from, unsigned char *to, long len)
    {
    unsigned char c,n;
    int w;
    unsigned char cur;
    long i=0;
    while (i<len)
        {
        cur=fgetc(from);
        if ((cur&0x80)!=0)
            {
            n=cur&0x7f;
            c=fgetc(from);
            }
            else
            {
            c=cur;
            n=1;
            }
        for (w=0; w<n; w++)
            to[i++]=c;
        }
    }

void encode_rcomp(unsigned char *from,FILE *to, long len)
    {
    unsigned char c,n;
    int w=0;
    unsigned char cur;
    long i;
    c=from[0];n=0;
    for (i=1; i<=len; i++)
        {
        if (i!=len) cur=from[i];
        n++;

        if (n==0x7f | c!=cur | i==len) w=1;

        if (w==1)
            {
            if (n>1) fputc(0x80|n,to);
            fputc(c,to);
            c=cur;n=0;w=0;
            }
        }
    }

void levellib::load(char *fname)
    {
    FILE *stream;
    stream=fopen(fname,"rb");
    int i;
    for (i=0; i<LEVEL_LIBSIZE; i++)
        {
        int x,y;
        x=fgetc(stream);
        y=fgetc(stream);
        mylevels[i]->resize(x,y);

        int xx,yy;
        unsigned char d[x*y];
        decode_rcomp(stream,d,x*y);
        for (yy=0; yy<y; yy++)
            for (xx=0; xx<x; xx++)
                mylevels[i]->set(xx,yy,d[xx+(yy*x)]);
        }
    fclose(stream);
    }

void levellib::save(char *fname)
    {
    FILE *stream;
    stream=fopen(fname,"wb");
    int i;
    for (i=0; i<LEVEL_LIBSIZE; i++)
        {
        int x,y;
        x=mylevels[i]->MX;
        y=mylevels[i]->MY;
        fputc(x,stream);
        fputc(y,stream);

        int xx,yy;
        unsigned char d[x*y];
        for (yy=0; yy<y; yy++)
            for (xx=0; xx<x; xx++)
                d[xx+(yy*x)]=mylevels[i]->get(xx,yy);
        encode_rcomp(d,stream,x*y);
        }
    fclose(stream);
    }

void levellib::make_lookup()
    {
    char *level_refs = " abcdefghijklmnopqrstuvwxyz";
    int i;
    for (i=0; i<256; i++)
        lookup[i]=0;
    for (i=0; i<LEVEL_LIBSIZE; i++)
        {
        lookup[level_refs[i]]=i;
        lookup[toupper(level_refs[i])]=i;
        }
    }


levellib::levellib(int mx, int my)
    {
    int j;
    make_lookup();
    for (j=0; j<LEVEL_LIBSIZE; j++)
        mylevels[j]=new pfield(1,1);
    levelresize(mx,my);
    }

levellib::levellib(char *fname)
    {
    int j;
    make_lookup();
    for (j=0; j<LEVEL_LIBSIZE; j++)
        mylevels[j]=new pfield(1,1);

    load(fname);
    }

void levellib::levelresize(int mx, int my)
    {
    int j;
    make_lookup();
    for (j=0; j<LEVEL_LIBSIZE; j++)
        mylevels[j]->resize(mx,my);
    }

pfield *levellib::level(unsigned char c)
    {
    int i=lookup[c];
    if (i<0 | i>=LEVEL_LIBSIZE)
        printf ("\a Error : %d too big a level ref\n",i);
        else
        {
        return(mylevels[i]);
        }
    }

levellib::~levellib()
    {
    int j;
    for (j=0; j<LEVEL_LIBSIZE; j++)
        delete mylevels[j];
    }


