#ifndef H_GUICLASSES
#define H_GUICLASSES
#include "../vid/mvid.h"
#include "../vid/dvid.h"

#include <stdio.h>


class sfont
    {
    public:
    int oMY;
    int MX,MY,bg,fg;
    unsigned char *font_data;

    sfont();
    sfont(int my);

    virtual void print_text(vid *myvid, char *s, int x, int y);
    virtual void print_text(vid *myvid, char *s, int x, int y, int mbg, int mfg);
    virtual int width(char *s);

    virtual ~sfont();
    };

class mouseevent
    {
    public:
        int x,y,b;
        mouseevent(int xx, int yy, int bb) {x=xx;y=yy;b=bb;}
        ~mouseevent() {}
    };

class keyevent
    {
    public:
        int k,ud;
        char ch;
        keyevent(int kk, char chch, int udud) {k=kk;ch=chch;ud=udud;}
        ~keyevent() {}
    };

#define FOCUS_UNREQUIRED 0
#define FOCUS_REQUIRED   1
#define FOCUS_REJECTED   2
#define REMOVE_NEVER 0
#define REMOVE_NOW   1
#define FOCUS_HAVENOT 0
#define FOCUS_HAVE    1

class component
    {
    public:
    long sMX, sMY, sPX, sPY;
    int MX, MY, PX, PY;

    component(long smx, long smy);
    void set_slocation(long spx, long spy);
    void set_location(int px, int py);

    virtual void resize(int mx, int my);
    virtual void resize();

    virtual int update(vid *onto); 
    virtual void mouse(mouseevent *e);
    virtual void key(keyevent *e);
    int require_focus;
    int remove_me;
    int have_focus;
    virtual ~component();
    };

class dbuff_component : public component
    {
    public:
    mvid *myvid;

    dbuff_component(long smx, long smy);
    virtual int update(vid *onto);
    virtual int update();
    virtual void resize(int mx, int my);
    virtual ~dbuff_component();
    };

class text_box : public dbuff_component
    {
    public:
    char *str; 
    sfont *fnt;
    text_box(sfont *f, char *s)
        : dbuff_component(f->width(s),f->MY)
        {str=s;fnt=f;}
    virtual void resize()
        {fnt->print_text(myvid,str,0,0);}
    };


#define MAX_COMPONENTS 128

#define PANEL_PERCENT  0x00010000
#define PANEL_ENTIRE   (PANEL_PERCENT|100)

#define PANEL_CENTER   0x00100000
#define PANEL_POINTS   0x0000ffff

/*
#define PANEL_LEFT     0x01000000
#define PANEL_TOP      0x02000000
#define PANEL_RIGHT    0x04000000
#define PANEL_BOTTOM   0x08000000

#define PANEL_SOMEPOS  0x0ff00000
  */

class panel : public component
    {
    public:
    int total_components;
    int panel_changes;
    component *focus;
    int omb;
    component *components[MAX_COMPONENTS];

    panel(long smx, long smy);
    void add(component *c, long spx, long spy);
    void add(component *c, long location);
    void remove(component *c);

    virtual int update(vid *onto);
    virtual void resize();

    void figure_out(component *c);

    virtual void mouse(mouseevent *e);
    virtual void key(keyevent *e);
    
    virtual ~panel();
    };

#define TOPWINDOW_MOUSE     1
#define TOPWINDOW_NO_DBUFF  2

class topwindow : public panel
    {
    public:
    dvid *mydvid;

    int myparams;
    topwindow(int params=0,int inwidth=0,int inheight=0);
    void run();
    virtual ~topwindow();
    };


class action_listener
    {
    public:
        action_listener() {}
        virtual void action(int a) {}
        virtual ~action_listener() {}
    };        

class popfocus_panel : public panel
    {
    public:
    popfocus_panel(long smx, long smy) : panel(smx,smy) { require_focus=FOCUS_REQUIRED; }
    virtual ~popfocus_panel() {}
    };

typedef struct {
char field[20];
int value;
} menuitem;

class menu_item : public text_box  // tells a listener when it was clicked
    {   
    public:
    action_listener *parent;

    int omb;
    menuitem *mi;

    menu_item(sfont *f, action_listener *p, menuitem *m);  
    void mouse(mouseevent *e);
    virtual ~menu_item();
    };

#define MAX_MENU_ITEMS 10
class vert_menu : public popfocus_panel
    {   //the menu that is pulled down..
    public:

    sfont *fnt;
    menuitem *mi;
    action_listener *parent;
    int total_hms;
    menu_item *hm[MAX_MENU_ITEMS];

    virtual int update(vid *onto);
    vert_menu(sfont *f,action_listener *p, menuitem *m, int msl, int hms);
    ~vert_menu();
    };

class pulldown_menu : public panel, public action_listener
    {
    public:

    panel *panl;
    action_listener *al;
    sfont *fnt;
    menuitem *mi;

    int total_ms;
    menu_item *hm[MAX_MENU_ITEMS];
    vert_menu *vm[MAX_MENU_ITEMS];

    virtual int update(vid *onto);
    pulldown_menu(sfont *f, panel *p, menuitem *m, action_listener *a);
    void action(int a);
    ~pulldown_menu();
    };

class edit_textbox : public dbuff_component
    {
    public:
    char *str;
    sfont *fnt;
    int changes;
    int cur;
    int len;
    int ohf;
    edit_textbox(sfont *f, char *s, int l);
    int update();
    void resize();
    virtual void key(keyevent *e);
    virtual ~edit_textbox();
    };

class edit_numberbox : public edit_textbox
    {
    public:
    int *num;
    edit_numberbox(sfont *f, int *n, int l);
    virtual void key(keyevent *e);
    virtual ~edit_numberbox();
    };

#endif
